/*
 * This file was generated by qdbusxml2cpp version 0.7
 * Command line was: qdbusxml2cpp -N -m -i generic-types.h -p nm-device-olpc-meshinterface /home/jgrulich/projects/libnm-qt/dbus/introspection/nm-device-olpc-mesh.xml
 *
 * qdbusxml2cpp is Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef NETWORKMANAGERQT_DEVICE_OLPC_MESHINTERFACE_H
#define NETWORKMANAGERQT_DEVICE_OLPC_MESHINTERFACE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>
#include "generic-types.h"

/*
 * Proxy class for interface org.freedesktop.NetworkManager.Device.OlpcMesh
 */
class OrgFreedesktopNetworkManagerDeviceOlpcMeshInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.freedesktop.NetworkManager.Device.OlpcMesh"; }

public:
    OrgFreedesktopNetworkManagerDeviceOlpcMeshInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = 0);

    ~OrgFreedesktopNetworkManagerDeviceOlpcMeshInterface();

    Q_PROPERTY(uint ActiveChannel READ activeChannel)
    inline uint activeChannel() const
    { return qvariant_cast< uint >(property("ActiveChannel")); }

    Q_PROPERTY(QDBusObjectPath Companion READ companion)
    inline QDBusObjectPath companion() const
    { return qvariant_cast< QDBusObjectPath >(property("Companion")); }

    Q_PROPERTY(QString HwAddress READ hwAddress)
    inline QString hwAddress() const
    { return qvariant_cast< QString >(property("HwAddress")); }

public Q_SLOTS: // METHODS
Q_SIGNALS: // SIGNALS
    void PropertiesChanged(const QVariantMap &properties);
};

#endif
